local getopt = require('getopt')
local utils = require('utils')
local ac = require('ansicolors')
line = '------------------------------------------------------------------------------'
mod = "            Modification for ELECTRA fob cloning SCRIPT lua - jareckib"
version = '                       version --- v1.1.2  18/07/2023'
desc = [[

   Cloning new ELECTRA tags to T5577 tags. This script modifies block 0 and 
   writes additional data to blocks 3 and 4. It keeps track of last 
   ELECTRA ID used. If press Y and <enter> it  which writes a ID.
   Any other input char will exit the script.

]]
example = [[
-------------------------------------------------------------------------------

---------------------- start cloning ELECTRA to T5577 -------------------------
  
  script run electra -s 11AA22BB55

---------------------- continue cloning last cloned ELECTRA ------------------- 
  
  script run electra -c
  
-------------------------------------------------------------------------------

]]
usage = [[
  script run electra.lua [-h] [-c] [-s <EM410x ID HEX number>]
]]
arguments = [[
    -h      : this help
    -c      : continue cloning from last ID used
    -s      : ELECTRA - EM410x ID HEX number
    ]]

-- Some globals
local DEBUG = false
local ELECTRA_STATUS_FN = 'electra_status_last_ID.txt'

-- A debug printout-function
local function dbg(args)
    if not DEBUG then return end
    if type(args) == 'table' then
        local i = 1
        while args[i] do
            dbg(args[i])
            i = i+1
        end
    else
        print('###', args)
    end
end

-- when errors occur
local function oops(err)
    print('ERROR:', err)
    core.clearCommandBuffer()
    return nil, errr
end

-- Usage help
local function help()
    core.console('clear')
    print(line)
    print(ac.cyan..mod..ac.reset)
    print(ac.cyan..version..ac.reset)
    print(ac.yellow..desc..ac.reset)
    print(ac.cyan..'  Usage'..ac.reset)
    print(usage)
    print(ac.cyan..'  Arguments'..ac.reset)
    print(arguments)
    print(ac.cyan..'  Example usage'..ac.reset)
    print(example)
end

-- Exit message
local function exitMsg(msg)
    print( string.rep('--',39) )
    print( string.rep('--',39) )
    print(msg)
    print()
end

local function readfile()
    local f = io.open(ELECTRA_STATUS_FN, "r")
    if f == nil then
        return nil, string.format("Could not read file %s", ELECTRA_STATUS_FN)
    end
    local t = f:read("*all")
    f:close()
    local cn_hi  = tonumber(t:sub(1, 2), 16)
    local cn_low = tonumber(t:sub(3, 10), 16)
    print(('       Using EM410x ID '..ac.green..'%02X%08X'..ac.reset..' from '..ac.yellow..'%s'..ac.reset..''):format(cn_hi, cn_low, ELECTRA_STATUS_FN))
    return cn_hi, cn_low
end

local function writefile(cn_hi, cn_low)
    local f = io.open(ELECTRA_STATUS_FN, "w")
    if f == nil then
        return nil, string.format("Could not write to file %s", ELECTRA_STATUS_FN)
    end
    f:write(("%02X%08X\n"):format(cn_hi, cn_low))
    f:close()
    print(('  Wrote EM410x ID '..ac.green..'%02X%08X'..ac.reset..' to `'..ac.yellow..'%s'..ac.reset..'`'):format(cn_hi, cn_low, ELECTRA_STATUS_FN))
    return true, 'Ok'
end

-- main
local function main(args)
    print( string.rep('--',39) )
    print( string.rep('--',39) )
    print()
    if #args == 0 then return help() end
    local shall_continue = false
    local startid = ''
	for o, a in getopt.getopt(args, 'cp:s:h') do
        if o == 'h' then return help() end
        if o == 'c' then shall_continue = true end
        if o == 's' then startid = a end
    end

    -- if start, check -s
    if not shall_continue then
        if startid == nil then return oops('empty EM410x ID string') end
        if #startid == 0 then return oops('empty EM410x ID string') end
        if #startid ~= 10 then return oops('EM410x ID wrong length. Must be 5 hex bytes') end
    end

    core.console('clear')
    print( string.rep('--',39) )
    print(ac.red..'                        DISABLE HINTS FOR LESS OUTPUT'..ac.reset)
    print( string.rep('--',39) )
    
    core.console('pref set hint --off')
    print('')
	local hi  = tonumber(startid:sub(1, 2), 16)
    local low = tonumber(startid:sub(3, 10), 16)
	if shall_continue then
        print('       Continue ELECTRA cloning from last save')
        hi, low = readfile()
    else
        print('  Starting ELECTRA cloning from entered EM-ELECTRA ID')
    end
	local template = 'EM410x ID '..ac.green..'%02X%08X'..ac.reset
    for i = low, low + 100, 1 do
        print('')
        print( string.rep('--',39) )
        local msg = (template):format(hi, low)
        print( string.rep('--',39) )
        print(('  %s'..ac.reset..' ELECTRA cloning to T5577? -'..ac.yellow..' y'..ac.reset..' for yes or '..ac.yellow..'enter'..ac.reset..' for exit'):format(msg))
    local ans = utils.input( ' ')
        if ans == 'y' then
            core.console('clear')
            print( string.rep('--',39) )
            core.console( ('lf em 410x clone --id %02X%08X'):format(hi, low) )
            core.console('lf t55 write -b 0 -d 00148080')
            core.console('lf t55 write -b 3 -d 7E1EAAAA')
            core.console('lf t55 write -b 4 -d AAAAAAAA')
        else
            core.console('clear')
            print( string.rep('--',39) )
            print(ac.red..'                              USER ABORTED'..ac.reset)
            print( string.rep('--',39) )
            break
        end
    end
    writefile(hi, low)
	print('  Enabling hints again')
    core.console('pref set hint --on')
    print( string.rep('--',39) )
    print( string.rep('--',39) )
end
main(args)

